import java.io.File;

import quicktime.QTSession;
import quicktime.std.movies.Movie;
import quicktime.io.QTFile;
import quicktime.io.IOConstants;
import quicktime.QTException;
import quicktime.std.StdQTException;
import quicktime.std.movies.media.DataRef;
import quicktime.std.StdQTConstants4;

public class ConcatenateMovieSegments {
	public static void main(String[] args) {
		if (args.length < 2) {
			System.out.println("usage: java ConcatenateMovieSegments <destination file> <source files>");
			return;
		}
		String destFilename = args[0];
		String[] srcFilenames = new String[args.length-1];
		System.arraycopy(args, 1, srcFilenames, 0, args.length-1);
		
		System.out.println("Locating segments...");
		File[] srcFiles = new File[srcFilenames.length];
		for (int i=0, n=srcFilenames.length; i<n; i++) {
			srcFiles[i] = new File(srcFilenames[i]);
			if (!srcFiles[i].exists()) {
				System.out.println("not found: "+srcFilenames[i]);
				return;
			}
		}
		
		// Initialize QuickTime
		try {
			QTSession.open();
		} catch (NoClassDefFoundError e) {
			System.out.println("This program requires QT4Java in order to run.");
			System.out.println("Download it from http://developer.apple.com/quicktime/qtjava/.");
			return;
		} catch (QTException e) {
			System.out.println("Unable to initialize QuickTime:");
			e.printStackTrace(System.out);
			return;
		}
		
		
		try {
			try {
				System.out.println("Opening segments...");
				Movie[] srcMovies = new Movie[srcFiles.length];
				try {
					for (int i=0, n=srcFiles.length; i<n; i++) {
						srcMovies[i] = Movie.fromDataRef(
							new DataRef(new QTFile(srcFiles[i])),
							StdQTConstants4.newMovieAsyncOK);
					}
					
					Movie dstMovie = new Movie();	// empty movie
					
					System.out.println("Concatenating segments...");
					for (int i=0, n=srcMovies.length; i<n; i++) {
						appendToEndOfMovie(srcMovies[i], dstMovie);
					}
					
					System.out.println("Saving movie file...");
					dstMovie.convertToFile(
						new QTFile(new File(destFilename)),
						(((int) 'M') << 24) | (((int) 'o') << 16) | (((int) 'o') << 8) | (((int) 'V') << 0),
						(((int) '?') << 24) | (((int) '?') << 16) | (((int) '?') << 8) | (((int) '?') << 0),
						IOConstants.smSystemScript);
				} finally {
					System.out.println("Closing segments...");
					for (int i=0, n=srcMovies.length; i<n; i++) {
						try {
							if (srcMovies[i] != null) {
								srcMovies[i].disposeQTObject();
							}
						} catch (QTException e) {
							// ignore errors while closing files
						}
					}
				}
			} catch (QTException e) {
				e.printStackTrace(System.out);
				return;
			}
		} finally {
			// Uninitialize QuickTime
			QTSession.close();
		}
	}
	
	private static void appendToEndOfMovie(Movie srcMovie, Movie dstMovie) throws StdQTException {
		// Select the end of the destination movie
		dstMovie.setSelection(dstMovie.getDuration(), 0);		// XXX: equiv: goToEnd() ???
		
		// Paste the source movie into the destination movie at its selection
		dstMovie.addSelection(srcMovie);
	}
}